try:
    from threading import local
except ImportError:
    from django.utils._threading_local import local

from cased_django import set_ip_address


class CasedIpMiddleware:
    def __init__(self, get_response):
        set_ip_address(None)
        self.get_response = get_response

    def _client_ip(self, request):
        x_forwarded_for = request.META.get("HTTP_X_FORWARDED_FOR")
        if x_forwarded_for:
            ip = x_forwarded_for.split(",")[0]
        else:
            ip = request.META.get("REMOTE_ADDR")
        return ip

    def __call__(self, request):
        ip = self._client_ip(request)
        set_ip_address(ip)
        response = self.get_response(request)
        return response

    def process_exception(self, request, exception):
        set_ip_address(None)
        return None



import cased
from cased_django.conf import (
    publish_key,
    api_base,
    publish_base,
    disable_publishing,
    reliability_backend,
    sensitive_data_handlers,
    sensitive_fields,
    log_level,
)


def get_cased_client():
    cased.publish_key = publish_key
    cased.api_base = api_base
    cased.publish_base = publish_base
    cased.disable_publishing = disable_publishing
    cased.reliability_backend = reliability_backend
    cased.sensitive_data_handlers = sensitive_data_handlers
    cased.sensitive_fields = sensitive_fields
    cased.log_level = log_level
    return cased


cased_client = get_cased_client()



import cased
from django.conf import settings

publish_key = getattr(settings, "CASED_PUBLISH_KEY", None)

log_level = getattr(settings, "CASED_LOG_LEVEL", None)

api_base = getattr(settings, "CASED_API_BASE", cased.api_base)
publish_base = getattr(settings, "CASED_PUBLISH_BASE", cased.publish_base)

disable_publishing = getattr(settings, "CASED_DISABLE_PUBLISHING", False)
include_ip_address = getattr(settings, "CASED_INCLUDE_IP_ADDRESS", False)

reliability_backend = getattr(settings, "CASED_RELIABILITY_BACKEND", None)
sensitive_data_handlers = getattr(settings, "CASED_SENSITIVE_DATA_HANDLERS", [])
sensitive_fields = getattr(settings, "CASED_SENSITIVE_FIELDS", set())



from django.db.models.signals import class_prepared, post_save, post_delete

from cased_django import cased_client
from cased_django import CasedModelEvent
from django.dispatch import receiver


class EventManager(object):
    @classmethod
    def publish_to_cased(cls, event):
        return cased_client.Event.publish(event)

    @classmethod
    def publish_create(cls, instance):
        event = instance.make_cased_event(instance, "create")
        return cls.publish_to_cased(event)

    @classmethod
    def publish_update(cls, instance):
        event = instance.make_cased_event(instance, "update")
        return cls.publish_to_cased(event)

    @classmethod
    def publish_delete(cls, instance):
        event = instance.make_cased_event(instance, "delete")
        return cls.publish_to_cased(event)


# Signal receivers


@receiver(class_prepared)
def bind_model(sender, **kwargs):
    """
    If the prepared class is a subclass of CasedModelEvent, then connect
    post_save and post_delete signals to it.

    Returns nothing.
    """
    if issubclass(sender, (CasedModelEvent,)):
        post_save.connect(event_save, sender=sender)
        post_delete.connect(event_delete, sender=sender)


def event_save(sender, instance, created, raw, **kwargs):
    if created:
        # new record
        return EventManager.publish_create(instance)
    else:
        # existing record
        return EventManager.publish_update(instance)


def event_delete(sender, instance, **kwargs):
    return EventManager.publish_delete(instance)



from django.utils.text import slugify
from cased_django.conf import include_ip_address
from . import get_ip_address


class CasedModelEvent(object):
    """
    Inherit from CasedModelEvent on your models to get automatic sending of
    audit entries to Cased.
    """

    @property
    def cased_payload(self):
        """
        Override with a dict to add any data you would like to send to Cased.

        Returns a dictionary
        """
        return {}

    @property
    def cased_actor(self):
        """
        Convenience function to automatically generate an actor with a standardized
        format. Subclass this in your custom model. For example

            class MyModel(models.Model, CasedModelEvent):
                author = ....

            @property
            def cased_actor(self):
                return self.author

        Returns a model instance.
        """
        return None

    @property
    def cased_event_actor(self):
        actor = self.cased_actor
        klass_name = actor.__class__.__name__

        pk = str(actor.pk)
        return klass_name + ";" + pk

    @property
    def cased_class_name(self):
        return slugify(self.__class__.__name__)

    def cased_event_action(self, event_type):
        return self.cased_class_name + "." + event_type

    def make_cased_event(self, instance, event_type):
        cased_payload = self.cased_payload

        if self.cased_actor:
            actor = self.cased_event_actor
            cased_payload["actor"] = actor

        if include_ip_address:
            cased_payload["location"] = get_ip_address()

        action = cased_payload.pop("action", None)
        if not action:
            # Auto-generate the action
            action = self.cased_event_action(event_type)

        event = dict(action=action, **cased_payload)

        return event



from threading import local

version = "0.3.7"

cased_thread_locals = local()


def get_ip_address():
    return getattr(cased_thread_locals, "ip_address", None)


def set_ip_address(ip):
    setattr(cased_thread_locals, "ip_address", ip)


from cased_django import conf
from .client import cased_client
from .middleware import CasedIpMiddleware
from .cased_model_event import CasedModelEvent
from .event_manager import EventManager



import os
import sys
from setuptools import setup, find_packages
from setuptools.command.test import test as TestCommand


setup(
    name="cased-django",
    version="0.3.7",
    description="Django integration for Cased",
    author="Cased",
    author_email="support@cased.com",
    url="https://github.com/cased/cased-django",
    license="Apache",
    keywords="cased api",
    packages=["cased_django"],
    zip_safe=False,
    python_requires=">3.5",
    project_urls={
        "Bug Tracker": "https://github.com/cased/cased-django/issues",
        "Source Code": "https://github.com/cased/cased-django",
    },
)
